@@ -1,6 +1,5 @@
 #  Net::Patricia - Patricia Trie perl module for fast IP address lookups
-#  Copyright (C) 2000-2005  Dave Plonka
-#  Copyright (C) 2009       Dave Plonka & Philip Prindeville
+#  Copyright (C) 2000  Dave Plonka
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,299 +15,52 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
+# $Id: Patricia.pm,v 1.8 2000/10/18 18:44:26 dplonka Exp $
 # Dave Plonka <plonka@doit.wisc.edu>
-# Philip Prindeville <philipp@redfish-solutions.com>
 
 package Net::Patricia;
 
 use strict;
-use warnings;
-
-require 5.008;
-
-use version;
 use Carp;
-use vars qw($VERSION @ISA @EXPORT);
-use Socket qw(AF_INET AF_INET6);
+use vars qw($VERSION @ISA @EXPORT_OK $AUTOLOAD);
 
-BEGIN {
-  require Exporter;
-  require DynaLoader;
-  @ISA = qw(Exporter DynaLoader);
-  @EXPORT = qw(AF_INET AF_INET6);
+require Exporter;
+require DynaLoader;
+require AutoLoader;
+
+@ISA = qw(Exporter DynaLoader);
+# Items to export into callers namespace by default. Note: do not export
+# names by default without a very good reason. Use EXPORT_OK instead.
+# Do not simply export all your public functions/methods/constants.
+@EXPORT_OK = ();
+'$Revision: 1.8 $' =~ m/(\d+)\.(\d+)/ && (( $VERSION ) = sprintf("%d.%03d", $1, $2));
+
+sub AUTOLOAD {
+    # This AUTOLOAD is used to 'autoload' constants from the constant()
+    # XS function.  If a constant is not found then control is passed
+    # to the AUTOLOAD in AutoLoader.
+
+    my $constname;
+    ($constname = $AUTOLOAD) =~ s/.*:://;
+    my $val = constant($constname, @_ ? $_[0] : 0);
+    if ($! != 0) {
+	if ($! =~ /Invalid/) {
+	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
+	    goto &AutoLoader::AUTOLOAD;
+	}
+	else {
+		croak "Your vendor has not defined Net::Patricia macro $constname";
+	}
+    }
+    eval "sub $AUTOLOAD { $val }";
+    goto &$AUTOLOAD;
 }
 
-'$Revision: 1.18_81 $' =~ m/(\d+)\.(\d+)((_\d+)|)/ && ( $VERSION = "$1.$2$3");
-
 bootstrap Net::Patricia $VERSION;
 
-sub new {
-  my ($class, $type) = @_;
+# Preloaded methods go here.
 
-  $type ||= AF_INET;
-
-  if ($type == AF_INET) {
-    return bless _new(32), 'Net::Patricia::AF_INET';
-  }
-
-  if ($type == AF_INET6) {
-    return bless _new(128), 'Net::Patricia::AF_INET6';
-  }
-
-  croak "new: unimplemented type";
-}
-
-##
-## Compat functions
-##
-
-sub _ip_bits {
-  my ($self, $str) = @_;
-  my $bits;
-
-  if (ref ($self) eq 'Net::Patricia::AF_INET6') { 
-	$bits = ($str =~ s|/(\d+)$||) ? $1 : 128; 
-  } else { 
-	$bits = ($str =~ s|/(\d+)$||) ? $1 : 32; 
-  }
-  ($str,$bits);
-}
-
-sub add_string {
-  croak "add_string: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self,$str,$data) = @_;
-  $data = $str unless @_ > 2;
-  $self->add($self->_ip_bits($str),$data);
-}
-
-sub match_string {
-  croak "match_string: wrong number of args" if (@_ != 2);
-  my ($self,$str) = @_;
-  $self->match($self->_ip_bits($str))
-}
-
-sub match_exact_string {
-  croak "match_exact_string: wrong number of args" if (@_ != 2);
-  my ($self,$str) = @_;
-  $self->exact($self->_ip_bits($str))
-}
-
-sub match_exact_integer {
-  shift->exact_integer(@_)
-}
-
-sub remove_string {
-  croak "remove_string: wrong number of args" if (@_ != 2);
-  my ($self,$str) = @_;
-  $self->remove($self->_ip_bits($str))
-}
-
-BEGIN {
-  eval {
-    my $class = 'Net::CIDR::Lite';
-    eval "require $class";
-  };
-  last if (@_);
-
-  sub add_cidr {
-    croak "add_cidr: wrong number of args" if (@_ != 3);
-    my ($self, $range, $data) = @_;
-    my $cidr = Net::CIDR::Lite->new();
-    $cidr->add_range($range);
-
-    my @list = ();
-    for ($cidr->list()) {
-      push(@list, $_) if ($self->add_string($_, $data));
-    }
-    @list;
-  }
-
-  sub remove_cidr {
-    croak "remove_cidr: wrong number of args" if (@_ != 2);
-    my ($self, $range) = @_;
-    my $cidr = Net::CIDR::Lite->new();
-    $cidr->add_range($range);
-
-    my @list = ();
-    for ($cidr->list()) {
-      push(@list, $_) if ($self->remove_string($_));
-    }
-    @list;
-  }
-}
-
-##
-## AF_INET
-##
-
-package Net::Patricia::AF_INET;
-
-use Carp;
-use Socket qw(AF_INET inet_aton inet_ntoa);
-use vars qw(@ISA @EXPORT);
-
-BEGIN {
-  require Exporter;
-  require DynaLoader;
-  @ISA = qw(Exporter DynaLoader Net::Patricia);
-  @EXPORT = qw(AF_INET);
-}
-
-sub add {
-  croak "add: wrong number of args" if (@_ < 2 || @_ > 4);
-  my ($self, $ip, $bits, $data) = @_;
-  $data = (defined $bits ? "$ip/$bits" : $ip) if (@_ < 4);
-  my $packed = inet_aton($ip);
-  croak("invalid key") unless (defined $packed);
-  $bits = 32 if (@_ < 3);
-  $self->SUPER::_add(AF_INET, $packed, $bits, $data);
-}
-
-sub add_integer {
-  croak "add_integer: wrong number of args" if (@_ < 2 || @_ > 4);
-  my ($self, $num, $bits, $data) = @_;
-  my $packed = pack("N", $num);
-  my $ip = inet_ntoa($packed);
-  croak("invalid address") unless (defined $ip);
-  $data = (defined $bits ? "$ip/$bits" : $ip) if (@_ < 4);
-  $bits = 32 if (@_ < 3);
-  $self->SUPER::_add(AF_INET, $packed, $bits, $data);
-}
-
-sub match_integer {
-  croak "match_integer: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $num, $bits) = @_;
-  $bits = 32 if (@_ < 3);
-  $self->SUPER::_match(AF_INET, pack("N",$num), $bits);
-}
-
-sub exact_integer {
-  croak "exact_integer: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $num, $bits) = @_;
-  $bits = 32 if (@_ < 3);
-  $self->SUPER::_exact(AF_INET, pack("N",$num), $bits);
-}
-
-sub match {
-  croak "match: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $ip, $bits) = @_;
-  my $packed = inet_aton($ip);
-  croak("invalid key") unless (defined $packed);
-  $bits = 32 if (@_ < 3);
-  $self->SUPER::_match(AF_INET, $packed, $bits);
-}
-
-sub exact {
-  croak "exact: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $ip, $bits) = @_;
-  my $packed = inet_aton($ip);
-  croak("invalid key") unless (defined $packed);
-  $bits = 32 if (@_ < 3);
-  $self->SUPER::_exact(AF_INET, $packed, $bits);
-}
-
-sub remove {
-  croak "remove: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $ip, $bits) = @_;
-  my $packed = inet_aton($ip);
-  croak("invalid key") unless (defined $packed);
-  $bits = 32 if (@_ < 3);
-  $self->SUPER::_remove(AF_INET, $packed, $bits);
-}
-
-sub remove_integer {
-  croak "remote_integer: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $num, $bits) = @_;
-  $bits = 32 if (@_ < 3);
-  $self->SUPER::_remove(AF_INET, pack("N",$num), $bits);
-}
-
-##
-## AF_INET6
-##
-
-package Net::Patricia::AF_INET6;
-
-use Carp;
-use Socket qw(AF_INET6);
-use Socket6 qw(inet_pton inet_ntop);
-use vars qw(@ISA @EXPORT);
-
-BEGIN {
-  require Exporter;
-  require DynaLoader;
-  @ISA = qw(Exporter DynaLoader Net::Patricia);
-  @EXPORT = qw(AF_INET6);
-}
-
-sub add {
-  croak "add: wrong number of args" if (@_ < 2 || @_ > 4);
-  my ($self, $ip, $bits, $data) = @_;
-  $data = (defined $bits ? "$ip/$bits" : $ip) if (@_ < 3);
-  my $packed = inet_pton(AF_INET6, $ip);
-  croak("invalid key") unless (defined $packed);
-  $bits = 128 if (@_ < 4);
-  $self->SUPER::_add(AF_INET6, $packed, $bits, $data);
-}
-
-sub add_integer {
-  croak "add_integer: wrong number of args" if (@_ < 2 || @_ > 4);
-  my ($self, $num, $bits, $data) = @_;
-  my $packed = pack("N", $num);
-  my $ip = inet_ntop(AF_INET6, $packed);
-  croak("invalid address") unless (defined $ip);
-  $data = (defined $bits ? "$ip/$bits" : $ip) if (@_ < 3);
-  $bits = 128 if (@_ < 4);
-  $self->SUPER::_add(AF_INET6, $packed, $bits, $data);
-}
-
-sub match_integer {
-  croak "match_integer: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $num, $bits) = @_;
-  $bits = 128 if (@_ < 3);
-  $self->SUPER::_match(AF_INET6, pack("N",$num), $bits);
-}
-
-sub exact_integer {
-  croak "exact_integer: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $num, $bits) = @_;
-  $bits = 128 if (@_ < 3);
-  $self->SUPER::_exact(AF_INET6, pack("N",$num), $bits);
-}
-
-sub match {
-  croak "match: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $ip, $bits) = @_;
-  my $packed = inet_pton(AF_INET6, $ip);
-  croak("invalid key") unless (defined $packed);
-  $bits = 128 if (@_ < 3);
-  $self->SUPER::_match(AF_INET6, $packed, $bits);
-}
-
-sub exact {
-  croak "exact: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $ip, $bits) = @_;
-  my $packed = inet_pton(AF_INET6, $ip);
-  croak("invalid key") unless (defined $packed);
-  $bits = 128 if (@_ < 3);
-  $self->SUPER::_exact(AF_INET6, $packed, $bits);
-}
-
-sub remove {
-  croak "remove: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $ip, $bits) = @_;
-  my $packed = inet_pton(AF_INET6, $ip);
-  croak("invalid key") unless (defined $packed);
-  $bits = 128 if (@_ < 3);
-  $self->SUPER::_remove(AF_INET6, $packed, $bits);
-}
-
-sub remove_integer {
-  croak "remote_integer: wrong number of args" if (@_ < 2 || @_ > 3);
-  my ($self, $num, $bits) = @_;
-  $bits = 128 if (@_ < 3);
-  $self->SUPER::_remove(AF_INET6, pack("N",$num), $bits);
-}
+# Autoload methods go after =cut, and are processed by the autosplit program.
 
 1;
 __END__
@@ -361,9 +113,11 @@ Routing Table for Berkeley Unix'' by Keith Sklower.
    $pt = new Net::Patricia;
 
 This is the class' constructor - it returns a C<Net::Patricia> object
-upon success or undef on failure.  The constructor takes an
-optional argument (of AF_INET or AF_INET6, defaulting to the former),
-and creates a tree with address and mask values of that type as keys.
+upon success or undef on failure.  For now, the constructor takes no
+arguments, and defaults to creating a tree which uses AF_INET IPv4
+address and mask values as keys.  In the future it will probably take
+one argument such as AF_INET or AF_INET6 to specify whether or not you
+are use 32-bit IP addresses as keys or 128-bit IPv6 addresses.
 
 The C<Net::Patricia> object will be destroyed automatically when
 there are no longer any references to it.
@@ -465,33 +219,7 @@ undef on failure.
    $pt->climb([CODEREF]);
 
 This method climbs the Patricia Trie, visiting each node as it does
-so.  It performs a non-recursive, "preorder" traversal.
-
-The CODEREF argument is optional.  It is a perl code reference used to
-specify a user-defined subroutine to be called when visiting each
-node.  The node's user data will be passed as the sole argument to that
-subroutine.
-
-This method returns the number of nodes successfully visited while
-climbing the Trie.  That is, without a CODEREF argument, it simply
-counts the number of nodes in the Patricia Trie.
-
-Note that currently the return value from your CODEREF subroutine is
-ignored.  In the future the climb method may return the number of times
-your subroutine returned non-zero, as it is called once per node.  So,
-if you are currently relying on the climb return value to accurately
-report a count of the number of nodes in the Patricia Trie, it would be
-prudent to have your subroutine return a non-zero value.
-
-This method is called climb() rather than walk() because climbing trees
-(and therfore tries) is a more popular pass-time than walking them.
-
-=item B<climb_inorder>
-
-   $pt->climb_inorder([CODEREF]);
-
-This method climbs the Patricia Trie, visiting each node in order as it
-does so.  That is, it performs an "inorder" traversal.
+so.
 
 The CODEREF argument is optional.  It is a perl code reference used to
 specify a user-defined subroutine to be called when visiting each
@@ -516,6 +244,29 @@ This method is called climb() rather than walk() because climbing trees
 
 =head1 BUGS
 
+I've only had the opportunity to test this code on GNU/Linux and
+Solaris.  [Why is it that I have fewer platforms available to me in
+academia than I used to have in the private industry?  Ugh.]  As such I
+am somewhat concerned about the portability of the C code on which this
+module is based and whether or not the resulting objects will link on
+other platforms.  Please send me reports, preferably including fixes
+for my Makefile.PL files and such.
+
+This is the first XSUB perl extension that I have written.  Consider
+yourself warned! ;^)  I'm particularly concerned as to whether or not
+my XS code is correct and whether or not this code leaks memory.  I've
+made an effort to avoid leaks in my use of the C patricialib API and in
+my manipulation of perl xVs, but am not sure if I have been successful
+in either case.  If leaks are discovered please report them to me as
+they are surely my fault and not that of the authors of the code on
+which this module is based.
+
+Methods to add or remove nodes using integer arguments are yet to be
+implemented.  This was a lower priority since it is less necessary to
+avoid the overhead involved in translation from a string representation
+since add and remove operations are usually performed less frequently
+than matching operations.
+
 This modules does not yet support AF_INET6 (IP version 6) 128 bit
 addresses, although the underlying patricialib C code does.
 
@@ -530,10 +281,8 @@ subroutine return a non-zero value.
 =head1 AUTHOR
 
 Dave Plonka <plonka@doit.wisc.edu>
-Philip Prindeville <philipp@redfish-solutions.com>
 
-Copyright (C) 2000-2005  Dave Plonka.  Copyright (C) 2009  Dave Plonka
-& Philip Prindeville.  This program is free software; you
+Copyright (C) 2000  Dave Plonka.  This program is free software; you
 can redistribute it and/or modify it under the terms of the GNU General
 Public License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
